import React, { useState, useEffect, useRef, useCallback } from "react";
import axios from "axios";
import {
  Box,
  Typography,
  Grid,
  Paper,
  Avatar,
  IconButton,
  Dialog,
  DialogContent,
  DialogTitle,
  DialogActions,
  FormControl,
  InputLabel,
  Select,
  MenuItem,
  TextField,
  TableContainer,
  Table,
  TableHead,
  TableRow,
  TableCell,
  TableBody,
  Button,
  TablePagination,
} from "@mui/material";
import { useParams, useNavigate } from "react-router-dom";
import { format } from "date-fns";
import { isAutheticated } from "../../auth";
import CancelIcon from "@mui/icons-material/Cancel"; // Add this import
import { debounce } from "lodash";
import InvoiceTable from "../orders/invoiceTable";
import PendingOrderTable from "../orders/pendingOrderTable";

const SingleRetailDistributor = () => {
  const { id } = useParams();
  const [retailerDetails, setRetailerDetails] = useState(null);
  const [openPopup, setOpenPopup] = useState(false);
  const [selectedImage, setSelectedImage] = useState("");
  const [orders, setOrders] = useState([]);
  const [totalOrders, setTotalOrders] = useState(0);
  const [page, setPage] = useState(0);
  const [rowsPerPage, setRowsPerPage] = useState(5);
  const [loading, setLoading] = useState(true);
  const [searchField, setSearchField] = useState("Order ID");
  const [searchText, setSearchText] = useState("");
  const token = isAutheticated();
  const navigate = useNavigate();
  const searchOrderIdRef = useRef("");
  const searchStatusRef = useRef("");
  // Debounced function to fetch orders
  const fetchOrdersDebounced = useRef(
    debounce((page, limit, orderId, status) => {
      fetchOrders(page, limit, orderId, status);
    }, 500)
  ).current;

  const handleSearchFieldChange = (event) => {
    const newSearchField = event.target.value;
  setSearchField(newSearchField);

  setSearchText("");

  if (newSearchField === "Order ID") {
    searchStatusRef.current = "";
  } else {
    searchOrderIdRef.current = ""; 
  }
  };


// When search text is typed
const handleSearchChange = (event) => {
  setSearchText(event.target.value);
  if (searchField === "Order ID") {
    searchOrderIdRef.current = event.target.value;
  } else {
    searchStatusRef.current = event.target.value;
  }

  // Reset page to 0 and fetch orders with the new search term
  setPage(0);
  fetchOrdersDebounced(1, rowsPerPage, searchOrderIdRef.current, searchStatusRef.current);
};
  const [openOrderModal, setopenOrderModal] = useState(false);
  const [singleorder, setSingleOrder] = useState(null);

  const handleCloseTMModal = () => {
    setopenOrderModal(false);
    setSingleOrder(null); // Clear the order details when closing the modal
  };
  // Function to fetch order details
  const fetchOrderDetails = async (id) => {
    try {
      const response = await axios.get(
        `/api/get-single-placed-order-rd/${id}`,
        {
          headers: {
            Authorization: `Bearer ${token}`,
          },
        }
      );
      setSingleOrder(response.data?.singleOrder);
      setopenOrderModal(true);
    } catch (error) {
      console.error("Error fetching order details:", error);
    }
  };
  const getUserDetails = useCallback(async () => {
    try {
      // Commented out the API call and using dummy data
      const response = await axios.get(`/api/getRD/${id}`, {
        headers: {
          "Access-Control-Allow-Origin": "*",
          Authorization: `Bearer ${token}`,
          "Content-Type": "multipart/form-data",
        },
      });
      setRetailerDetails(response.data);
      // console.log('Retailer Details: ', response.data);
    } catch (error) {
      console.error("Error fetching data: ", error);
    }
  }, [id, token]);
  // Fetch Orders with Pagination, Order ID, and Status search
  const fetchOrders = useCallback(
    async (page = 1, limit = rowsPerPage, orderId = "", status = "") => {
      setLoading(true);
      try {
        const response = await axios.get(`/api/single-rd-order/${id}`, {
          headers: { Authorization: `Bearer ${token}` },
          params: { page, limit, orderId, status },
        });
        setOrders(response.data.orders || []);
        setTotalOrders(response.data.totalOrders || 0);
      } catch (error) {
        swal({
          title: "Warning",
          text: error.message,
          icon: "error",
          button: "Close",
          dangerMode: true,
        });
      } finally {
        setLoading(false);
      }
    },
    [id, token, rowsPerPage]
  );

  useEffect(() => {
    fetchOrders(page + 1, rowsPerPage, searchOrderIdRef.current, searchStatusRef.current);
  }, [page, rowsPerPage]);

// Fetch retailer details on mount
useEffect(() => {
  getUserDetails();
}, [id, getUserDetails]);

  const handleOpenPopup = (imageUrl) => {
    setSelectedImage(imageUrl);
    setOpenPopup(true);
  };

  const handleClosePopup = () => {
    setOpenPopup(false);
    setSelectedImage("");
  };

  const handleCancel = () => {
    navigate("/retail-distributor");
  };

  if (!retailerDetails) {
    return <Typography>Loading...</Typography>;
  }

// Handle page change
const handleChangePage = (event, newPage) => {
  setPage(newPage);
};

// Handle rows per page change
const handleChangeRowsPerPage = (event) => {
  const newRowsPerPage = parseInt(event.target.value, 10);
  setRowsPerPage(newRowsPerPage);
  setPage(0); 
};
  return (
    <Box sx={{ p: 3 }}>
      <Box
        sx={{
          display: "flex",
          justifyContent: "space-between",
          alignItems: "center",
          mb: 3,
        }}
      >
        <Typography variant="h4">Retailer Details</Typography>
        <IconButton sx={{ color: "red" }} onClick={handleCancel}>
          Cancel <CancelIcon />
        </IconButton>
      </Box>

      <Paper sx={{ p: 2, mb: 3 }}>
        <Typography variant="h5" gutterBottom>
          Retailer Details
        </Typography>
        <Grid container spacing={2}>
          <Grid item xs={6}>
            <Typography>
              <strong>Trade Name:</strong> {retailerDetails.kyc.trade_name}
            </Typography>
            <Typography>
              <strong>Name:</strong> {retailerDetails.name}
            </Typography>
            <Typography>
              <strong>Address:</strong> {retailerDetails.kyc.address}
            </Typography>
            <Typography>
              <strong>Town/City:</strong> {retailerDetails.kyc.city}
            </Typography>
          </Grid>
          <Grid item xs={6}>
            <Typography>
              <strong>District:</strong> {retailerDetails.kyc.district}
            </Typography>
            <Typography>
              <strong>State:</strong> {retailerDetails.kyc.state}
            </Typography>
            <Typography>
              <strong>Pincode:</strong> {retailerDetails.kyc.pincode}
            </Typography>
            <Typography>
              <strong>Mobile Number:</strong> {retailerDetails.mobile_number}
            </Typography>
            <Typography>
              <strong>Mapped Principal Distributor:</strong>{" "}
              {retailerDetails?.principal_distributer?.name || "Not Mapped"}
            </Typography>
          </Grid>
        </Grid>
      </Paper>

      <Paper sx={{ p: 2, mb: 3 }}>
        <Typography variant="h6" gutterBottom>
          Documents
        </Typography>
        <Grid container spacing={2}>
          <Grid item xs={6}>
            <Typography>
              <strong>PAN Number:</strong> {retailerDetails.kyc.pan_number}
            </Typography>
            <Avatar
              variant="square"
              src={retailerDetails.kyc.pan_img.url}
              sx={{ width: 100, height: 100, mb: 2, cursor: "pointer" }}
              onClick={() => handleOpenPopup(retailerDetails.kyc.pan_img.url)}
            />
            <Typography>
              <strong>Aadhar Number:</strong>{" "}
              {retailerDetails.kyc.aadhar_number}
            </Typography>
            <Avatar
              variant="square"
              src={retailerDetails.kyc.aadhar_img.url}
              sx={{ width: 100, height: 100, mb: 2, cursor: "pointer" }}
              onClick={() =>
                handleOpenPopup(retailerDetails.kyc.aadhar_img.url)
              }
            />
            <Typography>
              <strong>GST Number:</strong> {retailerDetails.kyc.gst_number}
            </Typography>
            <Avatar
              variant="square"
              src={retailerDetails.kyc.gst_img.url}
              sx={{ width: 100, height: 100, mb: 2, cursor: "pointer" }}
              onClick={() => handleOpenPopup(retailerDetails.kyc.gst_img.url)}
            />
          </Grid>
          <Grid item xs={6}>
            <Typography>
              <strong>Pesticide License:</strong>
            </Typography>
            <Avatar
              variant="square"
              src={retailerDetails.kyc.pesticide_license_img.url}
              sx={{ width: 100, height: 100, mb: 2, cursor: "pointer" }}
              onClick={() =>
                handleOpenPopup(retailerDetails.kyc.pesticide_license_img.url)
              }
            />
            <Typography>
              <strong>Fertilizer License (optional):</strong>
            </Typography>
            {retailerDetails.kyc.fertilizer_license_img ? (
              <Avatar
                variant="square"
                src={retailerDetails.kyc.fertilizer_license_img.url}
                sx={{ width: 100, height: 100, mb: 2, cursor: "pointer" }}
                onClick={() =>
                  handleOpenPopup(
                    retailerDetails.kyc.fertilizer_license_img.url
                  )
                }
              />
            ) : (
              <Typography>Img not available</Typography>
            )}
            <Typography>
              <strong>Selfie of Entrance Board:</strong>
            </Typography>
            <Avatar
              variant="square"
              src={retailerDetails.kyc.selfie_entrance_img.url}
              sx={{ width: 100, height: 100, mb: 2, cursor: "pointer" }}
              onClick={() =>
                handleOpenPopup(retailerDetails.kyc.selfie_entrance_img.url)
              }
            />
          </Grid>
        </Grid>
      </Paper>

      <Paper sx={{ p: 2, mb: 3 }}>
        <Typography variant="h6" gutterBottom>
          Sales Coordinators/Territory Manager Details
        </Typography>
        <Grid container spacing={2}>
          <Grid item xs={6}>
            <Typography>
              <strong>Designation:</strong>{" "}
              {retailerDetails?.userType || "Not Available"}
            </Typography>
            <Typography>
              <strong>Name:</strong>{" "}
              {retailerDetails?.addedBy?.name || "Not Available"}
            </Typography>
            <Typography>
              <strong>ID:</strong>{" "}
              {retailerDetails?.addedBy?.uniqueId || "Not Available"}
            </Typography>
          </Grid>
        </Grid>
      </Paper>
      <Paper sx={{ p: 2, mb: 3 }}>
        {/* <div style={{ marginTop: "2rem" }}> */}
        <Typography variant="h5" gutterBottom>
          Orders
        </Typography>
        <Box className="mt-3">
          <Box display="flex" mb={2} alignItems="center">
            <FormControl variant="outlined" sx={{ minWidth: 150, mr: 2 }}>
              <InputLabel id="search-field-label">Search By</InputLabel>
              <Select
                labelId="search-field-label"
                id="search-field"
                value={searchField}
                onChange={handleSearchFieldChange}
                label="Search By"
              >
                <MenuItem value="Order ID">Order ID</MenuItem>
                <MenuItem value="Status">Status</MenuItem>
              </Select>
            </FormControl>
            <TextField
              label={`Search by ${searchField}`}
              variant="outlined"
              value={searchText}
              onChange={handleSearchChange}
              fullWidth
            />
          </Box>
          <TableContainer component={Paper}>
            <Table>
              <TableHead>
                <TableRow>
                  <TableCell>Order ID</TableCell>
                  <TableCell>Order Date</TableCell>
                  <TableCell>Items</TableCell>
                  <TableCell>Order Value</TableCell>
                  <TableCell>Status</TableCell>
                  <TableCell>Action</TableCell>
                </TableRow>
              </TableHead>
              <TableBody>
                {loading ? (
                  <TableRow>
                    <TableCell colSpan={6} align="center">
                      Loading...
                    </TableCell>
                  </TableRow>
                ) : orders.length === 0 ? (
                  <TableRow>
                    <TableCell colSpan={6} align="center">
                      No Orders Found
                    </TableCell>
                  </TableRow>
                ) : (
                  orders.map((order) => (
                    <TableRow key={order._id}>
                      <TableCell>{order.uniqueId}</TableCell>
                      <TableCell>
                        {new Date(order.createdAt).toLocaleString()}
                      </TableCell>
                      <TableCell>{order.orderItem.length}</TableCell>
                      <TableCell>₹ {order.grandTotal}</TableCell>
                      <TableCell>{order.status}</TableCell>
                      <TableCell>
                        <Button
                          variant="contained"
                          color="primary"
                          onClick={() => fetchOrderDetails(order._id)}
                        >
                          View
                        </Button>
                      </TableCell>
                    </TableRow>
                  ))
                )}
              </TableBody>
            </Table>
          </TableContainer>
          {/* Pagination */}
          <TablePagination
            rowsPerPageOptions={[5,10, 25, 50]}
            component="div"
            count={totalOrders}
            rowsPerPage={rowsPerPage}
            page={page}
            onPageChange={handleChangePage}
            onRowsPerPageChange={handleChangeRowsPerPage}
          />
          <Dialog
            open={openOrderModal}
            onClose={handleCloseTMModal}
            maxWidth="md"
            fullWidth
          >
            <DialogTitle>Order Details</DialogTitle>
            <DialogTitle>Order Id : {singleorder?.uniqueId}</DialogTitle>
            <DialogContent>
              {singleorder?.invoices?.length > 0 && (
                <>
                  <Typography variant="h4" gutterBottom>
                    Invoices
                  </Typography>
                  <InvoiceTable invoices={singleorder.invoices} />
                </>
              )}
              <Typography variant="h4" my={3} gutterBottom>
                Order Summary
              </Typography>

              <Grid container spacing={2}>
                <Grid item xs={12}>
                  <TableContainer component={Paper}>
                    <Table>
                      <TableHead>
                        <TableRow>
                          <TableCell>Product</TableCell>
                          <TableCell align="right">Price (₹)</TableCell>
                          <TableCell align="right">Quantity</TableCell>
                          <TableCell align="right">Subtotal (₹)</TableCell>
                          <TableCell align="right">GST (%)</TableCell>
                          <TableCell align="right">GST Amount (₹)</TableCell>
                          <TableCell align="right">
                            Total with GST (₹)
                          </TableCell>
                        </TableRow>
                      </TableHead>
                      <TableBody>
                        {singleorder?.orderItem.map((item, index) => {
                          const subtotal = item.price * item.quantity;
                          const gstAmount =
                            ((item.GST * item.price) / 100) * item.quantity;
                          const totalWithGST = subtotal + gstAmount;

                          return (
                            <TableRow key={index}>
                              <TableCell>
                                <img
                                  src={item.productId.image}
                                  alt={item.productId.name}
                                  style={{
                                    width: 50,
                                    height: 50,
                                    marginRight: 10,
                                  }}
                                />
                                <Typography variant="subtitle1">
                                  {item.productId.name}
                                </Typography>
                              </TableCell>
                              <TableCell align="right">₹{item.price}</TableCell>
                              <TableCell align="right">
                                {item.quantity}
                              </TableCell>
                              <TableCell align="right">₹{subtotal}</TableCell>
                              <TableCell align="right">{item.GST}%</TableCell>
                              <TableCell align="right">₹{gstAmount}</TableCell>
                              <TableCell align="right">
                                ₹{totalWithGST}
                              </TableCell>
                            </TableRow>
                          );
                        })}
                      </TableBody>
                    </Table>
                  </TableContainer>
                </Grid>
              </Grid>
              {singleorder?.invoices?.length > 0 && (
                <>
                  <Typography variant="h4" my={3} gutterBottom>
                    Order Items{" "}
                    {singleorder?.status == "pending"
                      ? "to be Processed"
                      : "Cancelled"}
                  </Typography>
                  <PendingOrderTable order={singleorder} />
                </>
              )}
            </DialogContent>
            <DialogActions>
              <Button onClick={handleCloseTMModal} color="secondary">
                Cancel
              </Button>
            </DialogActions>
          </Dialog>
        </Box>
        {/* </div> */}
      </Paper>
      <Dialog
        open={openPopup}
        onClose={handleClosePopup}
        maxWidth="md"
        fullWidth
      >
        <DialogTitle>
          <IconButton
            sx={{ position: "absolute", right: 16, top: 16, color: "red" }}
            onClick={handleClosePopup}
          >
            <CancelIcon />
          </IconButton>
        </DialogTitle>
        <DialogContent
          sx={{
            display: "flex",
            justifyContent: "center",
            alignItems: "center",
            p: 0,
            overflow: "auto",
            maxHeight: "80vh",
            maxWidth: "80vw",
          }}
        >
          <img
            src={selectedImage}
            alt="Large Preview"
            style={{
              maxWidth: "100%",
              maxHeight: "100%",
              objectFit: "contain",
            }}
          />
        </DialogContent>
      </Dialog>
    </Box>
  );
};

export default SingleRetailDistributor;
